import { Component, Input, OnInit } from '@angular/core';
import { AttendanceService } from 'app/features/attendances/services/attendance.service';
import { CourseUnitEvent } from '../../../courses/models/course-unit-event';
import * as moment from 'moment';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil } from 'rxjs/operators';
import { NotificationService } from 'app/shared/services/notification.service';
import { ParticipantAttendance } from 'app/features/attendances/dto/participant-attendance.dto';
import { BackendError } from 'app/shared/models/backend-error';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';

@Component({
  selector: 'ngx-course-participant-attendance',
  templateUrl: './course-participant-attendance.component.html',
  styleUrls: ['./course-participant-attendance.component.scss']
})
export class CourseParticipantAttendanceComponent extends BaseComponent implements OnInit {
  @Input() participantAttendance: ParticipantAttendance;
  @Input() writable: boolean = false;
  @Input() principal: SubatoPrincipal;

  processingEvent: CourseUnitEvent | null = null;

  constructor(
    private attendanceService: AttendanceService,
    private notificationService: NotificationService
  ) {
    super();
  }

  ngOnInit(): void {
  }

  get isLecturerOrAdmin() {
    return this.principal.isLecturerOrAdmin(this.participantAttendance.courseUnit.course.id);
  }

  get events() {
    return this.participantAttendance.events
      //.filter(e => moment().isSameOrAfter(e.takesPlaceAt))
      .sort((a, b) => a.takesPlaceAt.diff(b.takesPlaceAt));
  }

  isPresent(event: CourseUnitEvent): boolean {
    return this.participantAttendance.eventAttendances.find(eA => eA.event.id === event.id) != null;
  }

  createAttendance(event: CourseUnitEvent) {
    this.processingEvent = event;
    this.attendanceService.createAttendance({
      userId: this.participantAttendance.user.id,
      eventId: event.id,
    }).pipe(takeUntil(this.unsubscribe))
      .subscribe(attendance => {
        let eventAttendance = this.participantAttendance.eventAttendances.find(eA => eA.event.id === event.id);
        if(eventAttendance == null) {
          this.participantAttendance.eventAttendances.push(attendance);
          this.participantAttendance.presentSum += 1;
        }
        this.processingEvent = null;
        this.notificationService.showSuccess("Anwesenheit erfolgreich gesetzt");
      }, (error: any) => {
        this.processingEvent = null;

        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler beim Speichern der Anwesenheit",
            )
          );
        } else {
          throw error;
        }

      });
  }

  deleteAttendance(event: CourseUnitEvent) {
    this.processingEvent = event;
    this.attendanceService.deleteAttendance(this.participantAttendance.user.id, event.id)
    .pipe(takeUntil(this.unsubscribe))
      .subscribe(() => {
        let idx = this.participantAttendance.eventAttendances.findIndex(eA => eA.event.id === event.id);
        if(idx != -1) {
          this.participantAttendance.eventAttendances.splice(idx, 1);
          this.participantAttendance.presentSum -= 1;
        }
        this.processingEvent = null;
        this.notificationService.showSuccess("Anwesenheit erfolgreich entfernt");
      }, (error: any) => {
        this.processingEvent = null;
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler beim Speichern der Anwesenheit",
            )
          );
        } else {
          throw error;
        }
      });
  }
}
