import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Exercise } from 'app/features/exercises/models/exercise';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, Observable, of } from 'rxjs';
import { takeUntil, flatMap, catchError, filter, map } from 'rxjs/operators';
import { User } from 'app/core/models/user';
import { Submission } from '../../models/submission';
import { TaskInstance } from '../../models/task-instance';
import { BackendError } from 'app/shared/models/backend-error';
import { SubmissionService } from '../../services/submission.service';
import { NbDialogService } from '@nebular/theme';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { ParamHelper } from 'app/shared/helper/param.helper';

@Component({
  selector: 'ngx-exercise',
  templateUrl: './exercise.component.html',
  styleUrls: ['./exercise.component.scss']
})
export class ExerciseComponent extends BaseComponent implements OnInit {
  exercise: Exercise;
  submission: Submission;
  description: string;
  loading: boolean = true;
  principal: SubatoPrincipal;
  deletingExercise = false;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Übung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private exerciseService: ExerciseService,
    private submissionService: SubmissionService,
    private dialogService: NbDialogService,
    private appStateService: AppStateService,
    private router: Router,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  buildBreadcrumbs(exercise: Exercise) {
    this.breadcrumbs = [
      {
        id: 'exercises',
        params: {
          course_id: exercise.course.id
        },
      },
      {
        id: 'exercise',
        params: {
          exercise_id: exercise.id
        },
        ctx: exercise
      },
    ];
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let exerciseId = ParamHelper.parseIdOrThrow(params, "exercise_id");

          return forkJoin(
            this.exerciseService.getById(exerciseId),
            this.exerciseService.getDescription(exerciseId, false),
            this.loadSubmission(exerciseId)
          )
        }),
      ).subscribe(([exercise, description, submission]) => {
        this.exercise = exercise;
        this.description = description;
        this.submission = submission;
        this.appStateService.setCourse(exercise.course);
        this.buildBreadcrumbs(exercise);

        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  loadSubmission(exerciseId: number): Observable<Submission> {
    let loadSubmission = this.principal != null ?
      this.submissionService.getSubmissions({ exerciseId: exerciseId, userId: this.principal.id })
      : of(<Submission[]>[]);

    return loadSubmission.pipe(
      map(submissions => submissions[0]),
      flatMap(submission => submission ? this.submissionService.getById(submission.id) : of(null))
    )
  }

  getTaskInstanceLink(taskInstance: TaskInstance) {
    return ['/', 'exercises', this.exercise.id, 'tasks', taskInstance.task.id];
  }

  deleteExercise() {
    this.dialogService.open(DialogConfirmationComponent, {
      context: {
        title: `Löschung von Übung: ${this.exercise.name}`,
        text: `Bist du wirklich sicher, dass du die Übung '${this.exercise.name}' löschen willst? Alle verknüpften Lösungen, Abgaben etc. werden damit permanent gelöscht.`,
        actionText: 'Löschen'
      }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.confirmed),
      flatMap(() => {
        this.deletingExercise = true;
        return this.exerciseService.delete(this.exercise.id);
      })
    ).subscribe(() => {
      this.deletingExercise = false;
      this.router.navigate(['/', 'course', this.exercise.course.id, 'exercises'])
      this.notificationService.showSuccess("Übung erfolgreich gelöscht.");
    }, error => {
      this.deletingExercise = false;
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Löschen der Übung",
          )
        );
      } else {
        throw error;
      }
    })
  }
}
