import { AfterViewInit, Component, Input, OnInit, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { NbAutocompleteDirective } from '@nebular/theme';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { combineLatest, Observable, of, Subject, Subscription } from 'rxjs';
import { filter, flatMap, switchMap } from 'rxjs/operators';
import { debounceTime, takeUntil } from 'rxjs/operators';
import { Item } from '../../models/item';
import { SearchResult } from '../../models/search-result';
import { SearchService } from '../../services/search.service';
import * as _ from 'lodash';
import { AppStateService } from 'app/core/services/app.state.service';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { NotificationService } from 'app/shared/services/notification.service';
import { BackendError } from 'app/shared/models/backend-error';

class SearchItemGroup {
  group: string;
  items: Item[];
}

@Component({
  selector: 'search-form',
  templateUrl: './search-form.component.html',
  styleUrls: ['./search-form.component.scss']
})
export class SearchFormComponent extends BaseComponent implements OnInit {
  @Input() principal: SubatoPrincipal;
  @Input() courseId: number;

  selectedOption: any = null;
  result: SearchResult;
  itemGroups: SearchItemGroup[];

  @ViewChild(NbAutocompleteDirective) autocompleteDirective: NbAutocompleteDirective<any>;

  searching: boolean = false;
  searchingError: BackendError;
  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(
    private appStateService: AppStateService,
    private notificationService: NotificationService,
    private searchService: SearchService,
    private router: Router,
  ) {
    super();
  }

  get results(): Item[] {
    return this.result?.items ?? [];
  }

  get isStaff() {
    return this.principal != null && this.principal.hasAnyGroup(this.courseId)
  }

  ngOnInit(): void {
    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(100),
        switchMap(query => {
          this.searching = true;
          if(query) {
          return this.searchService.search(query, this.courseId);
          } else {
            return of(new SearchResult({ query: query, items: [] }));
          }
        })
      )
      .subscribe(result => {
        this.searching = false;
        this.result = result;

        let grouped = _.groupBy(result.items, item => item.group);
        this.itemGroups = Object.keys(grouped).map(group => ({ group: group, items: grouped[group] }));
      }, error => {
        this.searching = false;
        if (BackendError.isBackendError(error)) {
          this.searchingError = error;
          this.notificationService.showError(
            error.format(
              "Fehler bei der Suche",
            )
          );
        } else {
          throw error;
        }
      });

  }

  onChange(query) {
    this.query = query;

    if (typeof query == "string") {
      this.selectedOption = null;
      this.queryChanged.next(query);
    }
  }

  fn(option) {
    if (typeof option == "string") {
      return option
    } else {
      return  ""; //option.searchResult.query;
    }
  }
  onEnter(e) {
    /*
    if(!this.selectedOption) {
      this.router.navigate(['/', 'search'], {
        queryParams: {
          q: this.query
        }
      });
      this.autocompleteDirective.hide();
    }
    */
  }

  onSelectionChange(selectedValue) {
    if(selectedValue == null) return;
    this.selectedOption = selectedValue;
    this.router.navigate([this.selectedOption.url()], { queryParams: this.selectedOption.queryParams() });
    this.autocompleteDirective.hide();
  }
}
