package de.hsrm.sls.subato.solutions.evaluation.result;

import de.hsrm.sls.subato.solutions.evaluation.evaluator.Evaluator;

import jakarta.persistence.*;
import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

import java.util.ArrayList;
import java.util.List;

@Entity
@Table(name = "eval_result")
public class EvalResult {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @JdbcTypeCode(SqlTypes.LONGVARCHAR)
    private String runtimeOutput;

    private boolean compiling;

    @JdbcTypeCode(SqlTypes.LONGVARCHAR)
    private String compilationOutput;

    /**
     * how many malloc, calloc (etc.) operations were executed by the code. only applies to some languages, such as C
     */
    private int allocations;

    /**
     * how many free operations were executed by the code. only applies to some languages, such as C
     */
    private int frees;

    /**
     * how many test cases were executed. if the code can not be compiled, this value is 0
     */
    private int testsRun;

    /**
     * how many test cases failed with wrong assertions
     */
    private int testFailures;

    /**
     * how many test cases failed with errors.
     *
     * errors are distinguished from test failures: exceptions not being catched may lead to termination of the test,
     * so assertions cannot be checked.
     */
    private int testErrors;

    /**
     * more detailed information regarding execution of the test cases
     */
    @OneToMany(mappedBy = "evalResult", cascade = CascadeType.ALL)
    private List<TestCaseResult> testCaseResults = new ArrayList<>();

    /**
     * code style errors, each result maps to an error
     */
    @OneToMany(mappedBy = "evalResult", cascade = CascadeType.ALL)
    private List<StyleCheckResult> styleCheckResults = new ArrayList<>();

    @ManyToOne
    private Evaluator evaluator;

    public long getId() {
        return id;
    }

    public String getRuntimeOutput() {
        return runtimeOutput;
    }

    public boolean isCompiling() {
        return compiling;
    }

    public String getCompilationOutput() {
        return compilationOutput;
    }

    public int getTestErrors() {
        return testErrors;
    }

    public int getAllocations() {
        return allocations;
    }

    public int getFrees() {
        return frees;
    }

    public int getTestsRun() {
        return testsRun;
    }

    public int getTestFailures() {
        return testFailures;
    }

    public List<TestCaseResult> getTestCaseResults() {
        return testCaseResults;
    }

    public List<StyleCheckResult> getStyleCheckResults() {
        return styleCheckResults;
    }

    public void setRuntimeOutput(String runtimeOutput) {
        this.runtimeOutput = runtimeOutput;
    }

    public void setCompiling(boolean compilationFailed) {
        this.compiling = compilationFailed;
    }

    public void setCompilationOutput(String compilationOutput) {
        this.compilationOutput = compilationOutput;
    }

    public void setTestErrors(int errors) {
        this.testErrors = errors;
    }

    public void setAllocations(int allocations) {
        this.allocations = allocations;
    }

    public void setFrees(int frees) {
        this.frees = frees;
    }

    public void setTestsRun(int testsRun) {
        this.testsRun = testsRun;
    }

    public void setTestFailures(int testFailures) {
        this.testFailures = testFailures;
    }

    public void setTestCaseResults(List<TestCaseResult> testCaseResults) {
        this.testCaseResults = testCaseResults;
    }

    public void setStyleCheckResults(List<StyleCheckResult> styleCheckResults) {
        this.styleCheckResults = styleCheckResults;
    }

    public Evaluator getEvaluator() {
        return evaluator;
    }

    public void setEvaluator(Evaluator evaluator) {
        this.evaluator = evaluator;
    }

    /**
     * passed test cases are those, which could be executed without errors and who have no assertion errors
     */
    public int getTestsPassed() {
        return testsRun - testFailures - testErrors;
    }

    /**
     * metric for ranking results. ratio of #passed test cases / #total test cases.
     * if no tests were executed, then the correctness is 0 (todo: introduce null for this case)
     */
    public double getCorrectness() {
        return testsRun != 0 ? Math.round(getTestsPassed()/(double)testsRun*100) : 0;
    }

    /**
     * number of memory leaks
     */
    public int getLeaks() {
        return this.allocations - this.frees;
    }

    /**
     * automated rating for deciding which feedback to show
     */
    public EvalRating getRating() {
        var correctness = getCorrectness();
        if(!compiling || correctness < 50) {
            return EvalRating.BAD;
        }

        if(correctness >= 50 && correctness < 99) {
            return EvalRating.OK;
        }

        return EvalRating.GOOD;
    }
}
